/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail;

import java.rmi.Remote;
import java.rmi.RemoteException;

import jsjr.Connection;
import jsjr.ConnectorImpl;

/**
 * Represents basic manipulations on a space of RDF triples.
 * 
 * @author maxg
 */
public interface GraphSpaceBase extends Remote {
    
    /**
     * Initialize the graph space.
     * 
     * @param xml initial contents in XML
     * @return this
     * @throws RemoteException
     */
    public GraphSpaceBase initialize(String xml) throws RemoteException;
    
    /**
     * Add a triple to the graph space.
     * 
     * @param tuple triple to add
     * @return this
     * @throws RemoteException
     */
    public GraphSpaceBase add(Tuple tuple) throws RemoteException;
    
    /**
     * Remove a triple from the graph space.
     * 
     * @param tuple triple to remove
     * @return this
     * @throws RemoteException
     */
    public GraphSpaceBase remove(Tuple tuple) throws RemoteException;

}

class GraphSpaceBaseProxy implements GraphSpaceBase {
    
    private final Connection conn;
    private final String remoteId;
    
    public GraphSpaceBaseProxy(Connection conn, String remoteId) {
        this.conn = conn;
        this.remoteId = remoteId;
    }
    
    public GraphSpaceBase initialize(String xml) throws RemoteException {
        conn.call(remoteId, "initialize", ConnectorImpl.marshalable(xml));
        return this;
    }

    public GraphSpaceBase add(Tuple tuple) throws RemoteException {
        conn.call(remoteId, "add", tuple);
        return this;
    }
    
    public GraphSpaceBase remove(Tuple tuple) throws RemoteException {
        conn.call(remoteId, "remove", tuple);
        return this;
    }
}
