/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail;

import java.rmi.Remote;
import java.rmi.RemoteException;

import jsjr.Connection;

/**
 * A listener on graph spaces.
 * 
 * @author maxg
 */
public interface GraphSpaceListener extends Remote {
    
    /**
     * Called when an added triple triggers a result in a registered query.
     * 
     * @param solution query solution
     * @throws Exception
     */
    public void notify(Solution solution) throws Exception;

}

class GraphSpaceListenerProxy implements GraphSpaceListener {
    
    private final Connection conn;
    private final String remoteId;
    
    public GraphSpaceListenerProxy(Connection conn, String remoteId) {
        this.conn = conn;
        this.remoteId = remoteId;
    }

    public void notify(Solution solution) throws RemoteException {
        conn.call(remoteId, "notify", solution);
    }
}
