/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail;

import java.io.Serializable;
import java.util.*;

import jsjr.ConnectorImpl;
import jsjr.Marshalable;

/**
 * Represents a single answer from a SPARQL SELECT query.
 * 
 * @author maxg
 */
public class Solution extends HashMap<String, String> implements Marshalable {

    private static final long serialVersionUID = 1L;
    
    public Solution(String... keyValuePairs) {
        if (keyValuePairs.length % 2 != 0) throw new IllegalArgumentException();
        for (int ii = 0; ii < keyValuePairs.length; ii += 2) {
            put(keyValuePairs[ii], keyValuePairs[ii+1]);
        }
    }

    public String marshal() {
        StringBuilder messages = new StringBuilder();
        for (Map.Entry<String, String> param : entrySet()) {
            messages.append(ConnectorImpl.marshal(param.getKey()));
            messages.append(ConnectorImpl.marshal(param.getValue()));
        }
        messages.append("serial:class:codetrail.Solution&arguments:" + size()*2 + "\n");
        return messages.toString();
    }
}

class SolutionIterable implements Iterable<Solution>, Serializable {

    private static final long serialVersionUID = 1L;
    
    private final List<Solution> solutions;
    
    SolutionIterable() {
        solutions = new ArrayList<Solution>();
    }
    
    void add(Solution solution) {
        solutions.add(solution);
    }

    public Iterator<Solution> iterator() {
        return solutions.iterator();
    }
}