/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail;

import java.util.Iterator;

import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;

/**
 * Query solution utilities.
 * 
 * Methods are static utility methods rather than instance methods of
 * <code>Solution</code> so that <code>Solution</code>s do not depend on RDF
 * library classes.
 * 
 * @author maxg
 */
public abstract class SolutionUtil {
    
    /**
     * Construct a <code>Solution</code> equivalent to a
     * <code>QuerySolution</code>.
     * 
     * @param qsolution RDF API query solution
     * @return equivalent <code>Solution</code>
     */
    @SuppressWarnings("unchecked")
    public static Solution solution(QuerySolution qsolution) {
        Solution solution = new Solution();
        for (Iterator<String> vars = qsolution.varNames(); vars.hasNext(); ) {
            String var = vars.next();
            solution.put(var, qsolution.get(var) == null ? null : qsolution.get(var).toString());
        }
        return solution;
    }
    
    /**
     * Construct a <code>Solution</code> iterable over a set of
     * <code>QuerySolution</code>s.
     * 
     * @param results RDF API result set
     * @return equivalent iterable
     */
    public static Iterable<Solution> solutionIterable(ResultSet results) {
        SolutionIterable solutions = new SolutionIterable();
        while (results.hasNext()) {
            solutions.add(solution(results.nextSolution()));
        }
        return solutions;
    }
}
