/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail;

import jsjr.ConnectorImpl;
import jsjr.Marshalable;

/**
 * Represents a URI string.
 * 
 * @author maxg
 */
public class URIString implements Marshalable {
    
    private static final long serialVersionUID = 1L;
    
    public static final String BASE_URI = "http://uid.csail.mit.edu/graphspace";
    
    public static final String SEPARATOR = "#";
    
    /**
     * URI for <code>RDF:type</code>.
     */
    public static final URIString rdf_type = uri("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
    
    public static URIString uri(String uri) {
        return new URIString(uri);
    }
    
    public final String uri;
    
    public URIString(String uri) {
        if (uri.startsWith(SEPARATOR))
            this.uri = BASE_URI + uri;
        else
            this.uri = uri;
    }
    
    public String marshal() {
        return ConnectorImpl.marshal(uri)
                + "serial:class:codetrail.URIString&arguments:1\n";

    }
    
    @Override public boolean equals(Object o) {
        if ( ! (o instanceof URIString)) return false;
        URIString other = (URIString)o;
        return uri.equals(other.uri);
    }
    
    @Override public int hashCode() {
        return uri.hashCode() + 1;
    }
    
    @Override public String toString() {
        return uri;
    }
}
