/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse;

import static codetrail.eclipse.Activator.PLUGIN_ID;
import static codetrail.eclipse.Activator.activator;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Date;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

/**
 * Logging.
 * 
 * @author maxg
 */
public class Log {

    private static PrintStream log;
    static {
        try {
            log = new PrintStream(new FileOutputStream(activator().getStateLocation().append("log.log").toOSString(), true), true);
        } catch (Exception e) {
            log = System.err;
            try {
                activator().getLog().log(new Status(IStatus.ERROR, PLUGIN_ID, "Error establishing log", e));
            } catch (Exception f) { }
        }
    }

    /**
     * Log a trace.
     * 
     * @param message
     */
    public static void ok(String message) {
        log(IStatus.OK, message);
    }

    /**
     * Log information.
     * 
     * @param message
     */
    public static void info(String message) {
        log(IStatus.INFO, message);
    }

    /**
     * Log a warning.
     * 
     * @param message
     */
    public static void warn(String message) {
        log(IStatus.WARNING, message);
    }

    /**
     * Log an error.
     * 
     * @param message
     */
    public static void error(String message) {
        log(IStatus.ERROR, message);
    }

    /**
     * Log information with an exception.
     * 
     * @param message
     * @param error
     */
    public static void info(String message, Throwable error) {
        log(IStatus.INFO, message, error);
    }

    /**
     * Log a warning with an exception.
     * 
     * @param message
     * @param error
     */
    public static void warn(String message, Throwable error) {
        log(IStatus.WARNING, message, error);
    }

    /**
     * Log an error with an exception.
     * 
     * @param message
     * @param error
     */
    public static void error(String message, Throwable error) {
        log(IStatus.ERROR, message, error);
    }

    private static void log(int status, String message) {
        log.println(new Date() + ", " + message);
        if (status > IStatus.INFO) {
            activator().getLog().log(new Status(status, PLUGIN_ID, message));
        }
    }

    private static void log(int status, String message, Throwable error) {
        log.println(new Date() + ", " + message);
        error.printStackTrace(log);
        if (status > IStatus.INFO) {
            activator().getLog().log(new Status(status, PLUGIN_ID, message, error));
        }
    }
}
