/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.mod;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jface.action.*;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;

import codetrail.Solution;
import codetrail.eclipse.Activator;
import codetrail.eclipse.Log;
import codetrail.eclipse.study.Data;

/**
 * Bookmarks module menu contribution.
 * 
 * @author maxg
 */
public class BookmarksContrib extends CompoundContributionItem {

    public BookmarksContrib() {
    }

    public BookmarksContrib(String id) {
        super(id);
    }

    protected IContributionItem[] getContributionItems() {
        Bookmarks bookmarks = Activator.activator().module(Bookmarks.class);
        if (bookmarks == null) return new IContributionItem[0];
        
        List<IContributionItem> items = new ArrayList<IContributionItem>();
        for (Solution visit : bookmarks.visits) {
            URL icon = null;
            try {
                icon = new URL(visit.get("icon"));
            } catch (MalformedURLException murle) { }
            items.add(new ActionContributionItem(new BookmarkAction(visit.get("url"),
                                                                    visit.get("title"),
                                                                    icon)));
        }
        return items.toArray(new IContributionItem[items.size()]);
    }
}

class BookmarkAction extends Action implements IWorkspaceRunnable {
    
    private final String url, title;
    private IStructuredSelection selection;
    
    BookmarkAction(String url, String title, URL icon) {
        super(title, icon == null ? null : ImageDescriptor.createFromURL(icon)); // XXX icons too slow
        this.url = url;
        this.title = title;
    }
    
    @Override public void run() {
        Log.ok("Bookmark request from Eclipse");
        Data.ideEvent("bookmark req");
        selection = (IStructuredSelection)PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getSelection();
        try {
            ResourcesPlugin.getWorkspace().run(this, null);
        } catch (CoreException ce) {
            Log.error("Error running bookmarker", ce);
        }
    }
    
    public void run(IProgressMonitor monitor) {
        for (Iterator<?> it = selection.iterator(); it.hasNext(); ) {
            Object item = it.next();
            try {
                if (item instanceof IResource) {
                    Activator.activator().module(Bookmarks.class).bookmark(url, title, (IResource)item);
                } else if (item instanceof IJavaElement) {
                    Activator.activator().module(Bookmarks.class).bookmark(url, title, (IJavaElement)item);
                }
            } catch (Exception e) {
                Log.error("Error adding bookmark", e);
            }
        }
    }
}