/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.mod;

import static codetrail.Tuple.tuple;
import static codetrail.URIString.uri;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;

import org.eclipse.jface.text.*;
import org.eclipse.ui.texteditor.ITextEditor;

import codetrail.*;

/**
 * Element reporter module generic reporter.
 * 
 * @author maxg
 */
public class GenericReporter implements Reporter {
    
    private static final String QUERY = ReportElements.PROPERTIES.getProperty("generic-reporter-query");
    
    protected String selectedToken(ITextEditor editor, ITextSelection selection) throws BadLocationException {
        IDocument doc = editor.getDocumentProvider().getDocument(editor.getEditorInput());
        int lineOffset = doc.getLineOffset(selection.getStartLine());
        String[] tokens = doc.get(lineOffset, doc.getLineLength(selection.getStartLine())).split("\\b");
        
        String selected = null;
        int withinLineOffset = selection.getOffset() - lineOffset;
        for (String token : tokens) {
            withinLineOffset -= token.length();
            if (withinLineOffset <= 0 && token.matches("\\w+")) {
                selected = token;
                break;
            }
        }
        return selected;
    }
    
    public List<Tuple> selection(GraphSpace space, URIString event, ITextEditor editor, ITextSelection selection) throws BadLocationException, IOException {
        String token = selectedToken(editor, selection);
        if (token == null || token.length() < 3) return null;
        
        for (Solution solution : space.query(QUERY.replace("@TOKEN@", token.toLowerCase()))) { // XXX to lower case
            return selected(event, token, solution); // XXX returning first hit
        }
        
        return null;
    }
    
    private List<Tuple> selected(URIString event, String token, Solution doc) throws IOException {
        URIString eltURI = uri("#code-element-generic-" + URLEncoder.encode(token, "UTF-8"));
        String docURL = doc.get("template").replace(doc.get("replace"), token);
        return new ArrayList<Tuple>(Arrays.asList(
                tuple(event, uri("#artifact"), eltURI),
                tuple(eltURI, uri("#documentation"), docURL)
        ));
    }
}
