/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.study;

import static jsjr.ConnectorImpl.marshalable;

import java.io.*;
import java.rmi.ConnectException;
import java.util.Date;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import jsjr.*;
import codetrail.*;
import codetrail.eclipse.Activator;
import codetrail.eclipse.Log;

public class Data implements Module, Runnable, ConnectionListener {
    
    final static String DATA_ID = "@csail.mit.edu/codetrail/study/Data;1";
    
    final static String SAVE_FILE = "pending.data";
    
    private final BlockingQueue<Marshalable[]> events;
    private Thread thread;
    private Connection connection;
    
    public Data() {
        events = new LinkedBlockingQueue<Marshalable[]>();
    }
    
    public void connect(GraphSpace space) {
        try {
            ObjectInputStream in = new ObjectInputStream(new GZIPInputStream(new FileInputStream(saveFile())));
            @SuppressWarnings("unchecked")
            BlockingQueue<Marshalable[]> pending = (BlockingQueue)in.readObject();
            events.addAll(pending);
            saveFile().delete();
        } catch (Exception e) { }
                
        thread = new Thread(this, getClass().getName() + " connection");
        thread.setDaemon(true);
        thread.start();
        
        Activator.activator().connector().addConnectionListener(this); // XXX what if we already connected?
        
        ideEvent("start");
    }

    public void disconnect() {
        ideEvent("stop");
        
        thread.interrupt();
        try {
            thread.join();
        } catch (InterruptedException ie) { }
        thread = null;
        
        try {
            ObjectOutputStream out = new ObjectOutputStream(new GZIPOutputStream(new FileOutputStream(saveFile())));
            out.writeObject(events);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
            Log.error("Unable to save pending events", e);
        } finally {
            events.clear();
        }
    }

    public String getName() {
        return "User study data";
    }
    
    private File saveFile() {
        return Activator.activator().getStateLocation().append(SAVE_FILE).toFile();
    }
    
    public void connected(Connection conn) {
        connection = conn;
        try {
            connection.call(DATA_ID, "source", marshalable("eclipse"), marshalable(System.getProperty("osgi.framework.version")));
        } catch (Exception e) { }
    }
    
    public static void ideEvent(String event) {
        ideEvent(event, null, null);
    }
    
    public static void ideEvent(String event, String tag) {
        ideEvent(event, tag, null);
    }
    
    private static Data data = null;
    public static void ideEvent(String event, String tag, String url) {
        if (data == null)
            data = Activator.activator().module(Data.class);
        if (data == null)
            return;
        data.queueIdeEvent(event, tag, url);
    }
    
    private void queueIdeEvent(String event, String tag, String url) {
        events.offer(new Marshalable[] {
                marshalable(event),
                marshalable(tag),
                marshalable(url),
                marshalable(Util.sqlite().format(new Date()))
        });
    }
    
    private void send(Marshalable[] event) throws ConnectException {
        if (connection == null) throw new ConnectException("null");
        connection.call(DATA_ID, "ideEvent", event);
    }
    
    public void run() {
        Marshalable[] event;
        try {
            int seconds = 1;
            while( ! Thread.interrupted()) {
                event = events.take();
                try {
                    send(event);
                    seconds = 1;
                } catch (ConnectException ce) {
                    events.offer(event);
                    Thread.sleep(1000 * seconds);
                    if (seconds < 120) seconds *= 2;
                }
                Thread.sleep(500);
            }
        } catch (InterruptedException ie) { }
        try {
            while ( ! events.isEmpty()) {
                send(events.take());
            }
        } catch (Exception e) { }
    }
}
