/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.study;

import org.eclipse.ui.*;

import codetrail.GraphSpace;
import codetrail.Module;

public class Watcher implements Module, IWindowListener, IPageListener, IPartListener, IPerspectiveListener, IPropertyListener {
    
    public void connect(GraphSpace space) {
        PlatformUI.getWorkbench().addWindowListener(this);
        for (IWorkbenchWindow window : PlatformUI.getWorkbench().getWorkbenchWindows()) {
            windowOpened(window);
        }
    }

    public void disconnect() { }

    public String getName() {
        return "User study listener";
    }
    
    String indexOf(IWorkbenchWindow window) {
        IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
        for (int ii = 0; ii < windows.length; ii++) {
            if (windows[ii] == window) return Integer.toString(ii);
        }
        return "-1";
    }

    public void windowActivated(IWorkbenchWindow window) {
        Data.ideEvent("window active", indexOf(window));
    }

    public void windowClosed(IWorkbenchWindow window) {
        Data.ideEvent("window close", indexOf(window));
    }

    public void windowDeactivated(IWorkbenchWindow window) {
        Data.ideEvent("window deactive", indexOf(window));
    }

    public void windowOpened(IWorkbenchWindow window) {
        Data.ideEvent("window open", indexOf(window));
        window.addPageListener(this);
        window.addPerspectiveListener(this);
        for (IWorkbenchPage page : window.getPages()) {
            pageOpened(page);
        }
        perspectiveActivated(window.getActivePage(), window.getActivePage().getPerspective());
    }

    public void pageActivated(IWorkbenchPage page) { }

    public void pageClosed(IWorkbenchPage page) { }

    public void pageOpened(IWorkbenchPage page) {
        page.addPartListener(this);
        for (IWorkbenchPartReference part : page.getViewReferences()) {
            partOpened(part.getPart(false));
        }
        partActivated(page.getActivePart());
    }
    
    public void perspectiveActivated(IWorkbenchPage page, IPerspectiveDescriptor perspective) {
        Data.ideEvent("perspective", perspective.getLabel());
    }

    public void perspectiveChanged(IWorkbenchPage page, IPerspectiveDescriptor perspective, String changeId) { }

    public void partActivated(IWorkbenchPart part) {
        Data.ideEvent("part active", part.getClass().getSimpleName());
        part.addPropertyListener(this);
    }

    public void partBroughtToTop(IWorkbenchPart part) { }

    public void partClosed(IWorkbenchPart part) {
        Data.ideEvent("part close", part.getClass().getSimpleName());
    }

    public void partDeactivated(IWorkbenchPart part) { }

    public void partOpened(IWorkbenchPart part) {
        Data.ideEvent("part open", part == null ? "Unknown" : part.getClass().getSimpleName());
    }

    public void propertyChanged(Object source, int propId) {
        if (propId == IWorkbenchPartConstants.PROP_DIRTY && source instanceof ISaveablePart) {
            Data.ideEvent(((ISaveablePart)source).isDirty() ? "dirty" : "clean", source.getClass().getSimpleName());
        }
    }
}
