/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.mod;

import static codetrail.Tuple.tuple;
import static codetrail.URIString.uri;
import static org.junit.Assert.assertTrue;

import java.util.*;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.junit.*;

import codetrail.Tuple;
import codetrail.test.GraphSpaceStub;

public class ResourcesTest {
    
    private final IProgressMonitor monitor = new NullProgressMonitor();
    private IProject project;
    private Resources resources;
    private GraphSpaceStub space;
    
    @Before public void setUp() throws Exception {
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        project = root.getProject(this.getClass().getSimpleName() + "Project");
        
        project.create(monitor);
        project.open(monitor);
        
        resources = new Resources();
        space = new GraphSpaceStub();
        resources.connect(space);
    }

    @After public void tearDown() throws Exception {
        resources.disconnect();
        project.delete(true, monitor);
    }
    
    @Test public void testEmpty() throws InterruptedException {
        resources.notify(null);
        Thread.sleep(200);
        
        List<Tuple> expected = new ArrayList<Tuple>(Arrays.asList(new Tuple[] {
                tuple(uri("#r6e-%2F"), uri("#r6e-add"), "ResourcesTestProject")
        }));
        expected.removeAll(space.added);
        assertTrue("Missing: " + expected, expected.isEmpty());
    }
    
    @Test public void testCreate() throws CoreException, InterruptedException {
        IFolder folder = project.getFolder("foo");
        folder.create(IResource.NONE, false, null);
        IFile file = folder.getFile("bar.txt");
        file.create(null, IResource.NONE, null);
        Thread.sleep(200);
        
        String proj = "#r6e-%2FResourcesTestProject";
        List<Tuple> expected = new ArrayList<Tuple>(Arrays.asList(new Tuple[] {
                tuple(uri(proj), uri("#r6e-add"), "foo"),
                tuple(uri(proj + "%2Ffoo"), uri("#r6e-add"), "bar.txt"),
        }));
        expected.removeAll(space.added);
        assertTrue("Missing: " + expected, expected.isEmpty());
    }
    
    @Test public void testDelete() throws CoreException, InterruptedException {
        IFile file = project.getFile("baz.txt");
        file.create(null, IResource.NONE, null);
        Thread.sleep(200);
        
        String proj = "#r6e-%2FResourcesTestProject";
        List<Tuple> expected = new ArrayList<Tuple>(Arrays.asList(new Tuple[] {
                tuple(uri(proj), uri("#r6e-add"), "baz.txt"),
        }));
        expected.removeAll(space.added);
        assertTrue("Missing: " + expected, expected.isEmpty());
        
        file.delete(IResource.FORCE, null);
        Thread.sleep(200);
        
        assertTrue(space.added.contains(tuple(uri(proj), uri("#r6e-del"), "baz.txt")));
    }
}
