/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package codetrail.eclipse.mod;

import static codetrail.eclipse.mod.Snippets.SPLITTER;
import static org.junit.Assert.*;

import org.junit.*;

public class SnippetsTest {
    
    Snippets snippets;

    @Before public void setUp() {
        snippets = new Snippets();
    }

    @After public void tearDown() {
        snippets = null;
    }

    @Test public void testAlignChars() {
        assertEquals(
                new Snippets.Match(6, 4, 4, " ,m,n,o,p, ".split(","), " ,m,n,o,p, ".split(",")),
                Snippets.align("a,a,a,a, ,m,n,o,p, ,z,z,z,z".split(","),
                               "b,b,b,b, ,m,n,o,p, ,y,y,y,y".split(",")));
        
        assertEquals(
                new Snippets.Match(5, 2, 2, " ,m,n,o, ".split(","), " ,m,n,o, ".split(",")),
                Snippets.align("a,a, ,m,n,o, ,z,z,z,z,z, ,d,e,f,g".split(","),
                               "b,b, ,m,n,o, ,y,y,y,y,y, ,d,e,f,g".split(",")));
        
        assertEquals(
                new Snippets.Match(5, 1, 0, "a,b,c,1,2,!,3,d,e,f".split(","), "a,b,c,d,1,2,@,3,e,f".split(",")),
                Snippets.align("0,a,b,c,1,2,!,3,d,e,f".split(","),
                               "a,b,c,d,1,2,@,3,e,f,g".split(",")));
    }
    
    @Test public void testAlignCode() {
        String[] a, b;
        for (Code code : new Code[] {
                new Code(
                        a = "var foo = bar.someMethod(1, \"hello\", baz)".split(SPLITTER),
                        b = "var foo = bar.someMethod(2, 2, \"hello\", baz)".split(SPLITTER),
                        new Snippets.Match(6, 0, 0, a, b)),
        }) {
            assertEquals(code.match, Snippets.align(code.a, code.b));            
        }
    }
    
    static class Code {
        final String[] a;
        final String[] b;
        final Snippets.Match match;
        
        Code(String[] a, String[] b, Snippets.Match match) {
            this.a = a;
            this.b = b;
            this.match = match;
        }
    }
}
