/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

var data = Cc["@csail.mit.edu/codetrail/study/Data;1"].getService().wrappedJSObject;
var timeline;

function setup() {
  timeline = document.getElementById("timeline-tree");
  timeline.controllers.appendController({
    supportsCommand: function(cmd) { return cmd == "cmd_copy"; },
    isCommandEnabled: function(cmd) { return timeline.view.selection.getRangeCount() > 0; },
    doCommand: function(cmd) {
      var strings = [ ];
      var start = new Object();
      var end = new Object();
      var ranges = timeline.view.selection.getRangeCount();
      for (var range = 0; range < ranges; range++) {
        timeline.view.selection.getRangeAt(range, start, end);
        for (var idx = start.value; idx <= end.value; idx++) {
          strings.push(data.getSharedData(idx).join("\t"));
        }
      }
      Cc["@mozilla.org/widget/clipboardhelper;1"]
          .getService(Ci.nsIClipboardHelper)
              .copyString(strings.join("\n"));
    }
  });
  timeline.controllers.appendController({
    supportsCommand: function(cmd) { return cmd == "cmd_selectAll"; },
    isCommandEnabled: function(cmd) { return true; },
    doCommand: function(cmd) {
      timeline.view.selection.selectAll();
    }
  });
  refresh();
}

function refresh() {
  data.updateSharedData();
  timeline.view = {
    rowCount: data.getSharedCount(),
    getCellText: function(row, col) {
      return data.getSharedData(row)[col.index];
    },
    cycleHeader: function(col) { },
    setTree: function(treebox) { this.treebox = treebox; },
    isContainer: function(row) { return false; },
    isSeparator: function(row) { return false; },
    isSorted: function() { return false; },
    getLevel: function(row) { return 0; },
    getImageSrc: function(row, col) { return null; },
    getRowProperties: function(row, props) { },
    getCellProperties: function(row, col, props) { },
    getColumnProperties: function(colid, col, props) { }
  }
}

function debug() {
  document.getElementById("debug-button").style.display = "none";
  document.getElementById("debug").style.display = "";
}

function debugShareDomain() {
  data.shareDomain(document.getElementById("share-domain").value);
  document.getElementById("share-domain").value = "";
  refresh();
}

function debugShareUrl() {
  data.shareUrl(document.getElementById("share-url").value);
  document.getElementById("share-url").value = "";
  refresh();
}

function debugSubmit() {
  data.submit({ ok: "Codetrail data submitted.", fail: "Codetrail data submission failed." });
}
