/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "activator";
const THIS_CID = Components.ID("{@ACTIVATOR_CID@}");
const THIS_CONTRACTID = "@ACTIVATOR_CONTRACTID@";

Log = function() { return Cc["@csail.mit.edu/jsjr/log;1"].getService().wrappedJSObject; }

const ConnectorId = "@csail.mit.edu/jsjr/connector;1";

const StringBundleId = "@mozilla.org/intl/stringbundle;1";
const EnvironmentId = "@mozilla.org/process/environment;1";
const SocketTransportServiceId = "@mozilla.org/network/socket-transport-service;1";
const ScriptableInStreamId = "@mozilla.org/scriptableinputstream;1"
const InStreamPumpId = "@mozilla.org/network/input-stream-pump;1";
const UuidGenId = "@mozilla.org/uuid-generator;1";

function TupleEdge(subject, predicate, object) {
    this.subject = subject;
    this.predicate = predicate;
    this.object = object;
}

TupleEdge.prototype = {
    assert: function(rdf, source) {
        source.Assert(rdf.GetResource(this.subject.uri),
                      rdf.GetResource(this.predicate.uri),
                      rdf.GetResource(this.object.uri), true);
    },
    
    unassert: function(rdf, source) {
        source.Unassert(rdf.GetResource(this.subject.uri),
                        rdf.GetResource(this.predicate.uri),
                        rdf.GetResource(this.object.uri));
    },
    
    marshal: function() {
        var data = this.subject == null ? "null\n" : this.subject.marshal();
        data += this.predicate == null ? "null\n" : this.predicate.marshal();
        data += this.object == null ? "null\n" : this.object.marshal();
        data += "serial:class:codetrail.TupleEdge&arguments:3\n";
        return data;
    }
}

function TupleLeaf(subject, predicate, object) {
    this.subject = subject;
    this.predicate = predicate;
    this.object = object;
}

TupleLeaf.prototype = {
    assert: function(rdf, source) {
        source.Assert(rdf.GetResource(this.subject.uri),
                      rdf.GetResource(this.predicate.uri),
                      rdf.GetLiteral(this.object), true);
    },
    
    unassert: function(rdf, source) {
        source.Unassert(rdf.GetResource(this.subject.uri),
                        rdf.GetResource(this.predicate.uri),
                        rdf.GetLiteral(this.object));
    },
    
    marshal: function() {
        var data = this.subject == null ? "null\n" : this.subject.marshal();
        data += this.predicate == null ? "null\n" : this.predicate.marshal();
        data += this.object == null ? "null\n" : "str:" + this.object.replace(/\n/g, "\nlstr:") + "\n";
        data += "serial:class:codetrail.TupleLeaf&arguments:3\n";
        return data;
    }
}

function URIString(uri) {
    this.uri = uri;
}

URIString.prototype = {
    marshal: function() {
        var data = "";
        data += "str:" + this.uri + "\n";
        data += "serial:class:codetrail.URIString&arguments:1\n"
        return data;
    }
}

/*
 * Activator.
 */

function Activator() {
    this.wrappedJSObject = this;
}

Activator.prototype = {
    init: function() {
        //dump("Activator.init()\n");
        var env = Cc[EnvironmentId].getService(Ci.nsIEnvironment);
        this.CONNECTOR_PORT = (env.exists("GRAPHSPACE_CONNECTOR_PORT")
                               ? env.get("GRAPHSPACE_CONNECTOR_PORT")
                               : 3943);
        this.GRAPHSPACE_NAME = (env.exists("GRAPHSPACE_NAME")
                                ? env.get("GRAPHSPACE_NAME")
                                : "theGraphSpace");
        
        this.uuidgen = Cc[UuidGenId].getService(Ci.nsIUUIDGenerator);
        this.FIREFOX = new URIString("#firefox-" + this.uuidgen.generateUUID().toString());
        
        var strings = Cc[StringBundleId]
                          .getService(Ci.nsIStringBundleService)
                              .createBundle("chrome://"+EXTENSION_NAME+"/content/activator.properties");
        this.modules = strings.GetStringFromName("all-mods").split(/\s+/);
        
        this.connector = Cc[ConnectorId].getService().wrappedJSObject;
        
        this.connector.addListener(this);
        
        this.connector.addBuilder("codetrail.URIString", function(args) {
            return new URIString(args[0]);
        });
        this.connector.addBuilder("codetrail.TupleEdge", function(args) {
            return new TupleEdge(args[0], args[1], args[2]);
        });
        this.connector.addBuilder("codetrail.TupleLeaf", function(args) {
            return new TupleLeaf(args[0], args[1], args[2]);
        });
        this.connector.addBuilder("codetrail.Solution", function(args) {
            var map = { };
            for (var ii = 0; ii < args.length; ii += 2) {
                map[args[ii]] = args[ii+1];
            }
            return map;
        });
    },
    
    newWindow: function(window) {
        //dump("Activator.newWindow(" + window + ")\n");
        var content = window.document.getElementById("appcontent");
        var connector = this.connector;
        var port = this.CONNECTOR_PORT;
        content.addEventListener("load", function(e) { connector.asyncConnect(port); }, true);
        
        for each (module in this.modules) {
            Cc[module].getService().wrappedJSObject.newWindow(window);
        }
    },
    
    connect: function(connector) {
        //dump("Activator.connect(" + connector + ")\n");
        for each (module in this.modules) {
            Cc[module].getService().wrappedJSObject.connect(this);
        }
    },
    
    disconnect: function() {
        //dump("Activator.disconnect()\n");
        for each (module in this.modules) {
            Cc[module].getService().wrappedJSObject.disconnect();
        }
    },
    
    /*
     * Graph space access.
     */
    
    add: function(tuples) {
        this.connector.call(this.GRAPHSPACE_NAME,
                            "add",
                            [ "[Lcodetrail.Tuple;" ],
                            tuples);
    },
    
    register: function(listener, template, query) {
        this.connector.call(this.GRAPHSPACE_NAME,
                            "register",
                            [ "codetrail.GraphSpaceListener", "codetrail.Tuple", "java.lang.String" ],
                            [ listener, template, query ]);
    },
    
    registerBase: function(listener) {
        this.connector.call(this.GRAPHSPACE_NAME,
                            "register",
                            [ "codetrail.GraphSpaceBase" ],
                            [ listener ]);
    },
    
    registerAnswer: function(answerer, uri) {
        this.connector.call(this.GRAPHSPACE_NAME,
                            "register",
                            [ "codetrail.GraphSpaceListener", "codetrail.URIString" ],
                            [ answerer, uri ]);
    },
    
    answer: function(question, answer) {
        this.connector.call(this.GRAPHSPACE_NAME,
                            "answer",
                            [ "codetrail.URIString", "java.lang.String" ],
                            [ question, answer ]);
    },
    
    /*
     * Types.
     */
    
    Tuple: function(subject, predicate, object) {
        if (typeof(object) == "string") {
            return new TupleLeaf(subject, predicate, object);
        } else {
            return new TupleEdge(subject, predicate, object);
        }
    },
    
    Timestamp: function(subject) {
        return this.Tuple(subject,
                          this.URI("#timestamp"),
                          new Date().toLocaleFormat("%Y-%m-%dT%H:%M:%S"));
    },
    
    Unique: function(type) {
        return this.Tuple(this.URI(type + "-" + this.uuidgen.generateUUID().toString()),
                          this.RDF_TYPE,
                          this.URI(type));
    },
        
    URI: function(uri) {
        return new URIString(uri)
    },
    RDF_TYPE: new URIString("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"),
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new Activator();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}
