/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "RDF datasource";
const THIS_CID = Components.ID("{@DATASOURCE_CID@}");
const THIS_CONTRACTID = "@DATASOURCE_CONTRACTID@";

Log = function() { return Cc["@csail.mit.edu/jsjr/log;1"].getService().wrappedJSObject; }

const RdfServiceId =         "@mozilla.org/rdf/rdf-service;1";
const InMemoryDatasourceId = "@mozilla.org/rdf/datasource;1?name=in-memory-datasource";
const SimpleUriId =          "@mozilla.org/network/simple-uri;1";
const XmlParserId =          "@mozilla.org/rdf/xml-parser;1";

/*
 * RDF datasource.
 */

function DataSource() {
    this.wrappedJSObject = this;
    this.source = Cc[InMemoryDatasourceId].createInstance(Ci.nsIRDFDataSource);
    this.source.QueryInterface(Ci.nsIRDFPurgeableDataSource);
    this.rdf = Cc[RdfServiceId].getService(Ci.nsIRDFService);
}

DataSource.prototype = {
    connect: function(activator) {
        //dump("DataSource.connect(" + activator + ")\n");
        this.activator = activator;
        activator.registerBase(this);
    },
    
    disconnect: function() {
        delete this.activator;
        this.source.Sweep();
    },
    
    newWindow: function(window) { },
    
    /*
     * nsIRDFDataSource methods
     */
    /*void*/ AddObserver: function(/*nsIRDFObserver*/ observer) {
        //dump("DataSource.AddObserver(" + observer + ")\n");
        this.source.AddObserver(observer);
    },
    /*void*/ RemoveObserver: function(/*nsIRDFObserver*/ observer) {
        //dump("DataSource.RemoveObserver(" + observer + ")\n");
        this.source.RemoveObserver(observer);
    },
    /*nsISimpleEnumerator*/ ArcLabelsIn: function(/*nsIRDFNode*/ node) {
        //dump("DataSource.ArcLabelsIn(" + node + ")\n");
        return this.source.ArcLabelsIn(node);
    },
    /*nsISimpleEnumerator*/ ArcLabelsOut: function(/*nsIRDFResource*/ source) {
        //dump("DataSource.ArcLabelsOut(" + source.Value + ")\n");
        return this.source.ArcLabelsOut(source)
    },
    /*void*/ Assert: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        dump("DataSource.Assert(" + source.Value + ", " + property.Value + ", " + target + ", " + truthValue + ") unsupported\n");
    },
    /*void*/ beginUpdateBatch: function() {
        dump("DataSource.beginUpdateBatch() unsupported\n");
    },
    /*void*/ Change: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ oldTarget, /*nsIRDFNode*/ newTarget) {
        dump("DataSource.Change(" + source.Value + ", " + property.Value + ", " + oldTarget + ", " + newTarget + ") unsupported\n");
    },
    /*void*/ DoCommand: function(/*nsISupportsArray*/ sources, /*nsIRDFResource*/ command, /*nsISupportsArray*/ arguments) {
        dump("DataSource.DoCommand(" + sources + ", " + command.Value + ", " + arguments + ") unsupported\n");
    },
    /*void*/ endUpdateBatch: function() {
        dump("DataSource.endUpdateBatch() unsupported\n");
    },
    /*nsISimpleEnumerator*/ GetAllCmds: function(/*nsIRDFResource*/ source) {
        //dump("DataSource.GetAllCmds(" + source + ")\n");
        return this.source.GetAllCmds(source);
    },
    /*nsISimpleEnumerator*/ GetAllResources: function() {
        //dump("DataSource.GetAllResources()\n");
        return this.source.GetAllResources();
    },
    /*nsIRDFResource*/ GetSource: function(/*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("DataSource.GetSource(" + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.GetSource(property, target, truthValue);
    },
    /*nsISimpleEnumerator*/ GetSources: function(/*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("DataSource.GetSources(" + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.GetSources(property, target, truthValue);
    },
    /*nsIRDFNode*/ GetTarget: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*PRBool*/ truthValue) {
        //dump("DataSource.GetTarget(" + source.Value + ", " + property.Value + ", " + truthValue + ")\n");
        return this.source.GetTarget(source, property, truthValue);
    },
    /*nsISimpleEnumerator*/ GetTargets: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*PRBool*/ truthValue) {
        //dump("DataSource.GetTargets(" + source.Value + ", " + property.Value + ", " + truthValue + ")\n");
        return this.source.GetTargets(source, property, truthValue);
    },
    /*PRBool*/ hasArcIn: function(/*nsIRDFNode*/ node, /*nsIRDFResource*/ arc) {
        //dump("DataSource.hasArcIn(" + node + ", " + arc.Value + ")\n");
        return this.source.hasArcIn(node, arc);
    },
    /*PRBool*/ hasArcOut: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ arc) {
        //dump("DataSource.hasArcOut(" + source.Value + ", " + arc.Value + ")\n");
        return this.source.hasArcOut(source, arc);
    },
    /*PRBool*/ HasAssertion: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("DataSource.HasAssertion(" + source.Value + ", " + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.HasAssertion(source, property, target, truthValue);
    },
    /*PRBool*/ IsCommandEnabled: function(/*nsISupportsArray*/ sources, /*nsIRDFResource*/ command, /*nsISupportsArray*/ arguments) {
        //dump("DataSource.IsCommandEnabled(" + sources + ", " + command.Value + ", " + arguments + ")\n");
        return this.source.IsCommandEnabled(sources, command, arguments);
    },
    /*void*/ Move: function(/*nsIRDFResource*/ oldSource, /*nsIRDFResource*/ newSource, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target) {
        dump("DataSource.Move(" + oldSource.Value + ", " + newSource.Value + ", " + property.Value + ", " + target + ") unsupported\n");
    },
    /*void*/ Unassert: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target) {
        dump("DataSource.Unassert(" + source.Value + ", " + property.Value + ", " + target + ") unsupported\n");
    },
    
    initialize: function(xml) {
        //dump("DataSource.initialize(\n" + xml + "\n)\n");
        Cc[XmlParserId]
            .getService(Ci.nsIRDFXMLParser)
                .parseString(this.source, Cc[SimpleUriId].createInstance(Ci.nsIURI), xml);
    },
    
    add: function(tuple) {
        //dump("DataSource.add(" + tuple.subject.uri + ", " + tuple.predicate.uri + ", " + tuple.object + ")\n");
        tuple.assert(this.rdf, this.source);
    },
    
    remove: function(tuple) {
        //dump("DataSource.remove(" + tuple.subject.uri + ", " + tuple.predicate.uri + ", " + tuple.object + ")\n");
        tuple.unassert(this.rdf, this.source);
    },
    
    marshal: function() {
        return "remote:interface:codetrail.GraphSpaceBase&id:" + THIS_CONTRACTID + "\n";
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports) &&
             ! iid.equals(Ci.nsIRDFDataSource))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component.
 */
function SingletonFactory() {
    this.instance = new DataSource();
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}