/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "bookmarks integration";
const THIS_CID = Components.ID("{@MOD_BOOKMARKS_CID@}");
const THIS_CONTRACTID = "@MOD_BOOKMARKS_CONTRACTID@";

Log = function() { return Cc["@csail.mit.edu/jsjr/log;1"].getService().wrappedJSObject; }

const RdfServiceId = "@mozilla.org/rdf/rdf-service;1";
const NavBookmarksServiceId = "@mozilla.org/browser/nav-bookmarks-service;1";

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

const FOLDER_GUID = "41f23c00-1ba9-11dd-bd0b-0800200c9a66"; // XXX

/*
 * Firefox-Eclipse bookmarks integration.
 */

function ModBookmarks() {
    this.wrappedJSObject = this;
}

ModBookmarks.prototype = {
    connect: function(activator) {
        this.activator = activator;
        
        this.rdf = Cc[RdfServiceId].getService(Ci.nsIRDFService);
        this.datasource = this.rdf.GetDataSource("rdf:codetrail-eclipse-resources");
    },
    
    disconnect: function() {
        delete this.activator;
    },
    
    newWindow: function(window) {
        if (window.document.getElementById("bookmarks-menu")) {
            // MOZ Firefox 2
            window.document.getElementById("bookmarks-menu").datasources += " rdf:codetrail-graphspace";
            
            var cmd = "Cc[\""+THIS_CONTRACTID+"\"].getService().wrappedJSObject.bookmark(window, BROWSER_ADD_BM_FEATURES);";
            var menuSibling = window.document.getElementById("bookmarkAllCmd");
        } else {
            // MOZ Firefox 3
            
            // XXX show bookmarks in menu
            /*
            var bookmarks = Cc[NavBookmarksServiceId].getService(Ci.nsINavBookmarksService);
            var id = bookmarks.getItemIdForGUID(FOLDER_GUID);
            if (id < 0) {
                id = bookmarks.createDynamicContainer(bookmarks.toolbarFolder,
                                                      "Eclipse Bookmarks",
                                                      THIS_CONTRACTID,
                                                      Ci.nsINavBookmarksService.DEFAULT_INDEX);
                bookmarks.setItemGUID(id, FOLDER_GUID);
            }
            */
            
            var cmd = "Cc[\""+THIS_CONTRACTID+"\"].getService().wrappedJSObject.bookmark(window, null);";
            var menuSibling = window.document.getElementById("subscribeToPageMenupopup").nextSibling;
        }
        var bookmarkEclipse = window.document.createElementNS(XUL_NS, "menuitem");
        bookmarkEclipse.setAttribute("label", "Bookmark This Page in Eclipse...");
        bookmarkEclipse.setAttribute("oncommand", cmd);
        menuSibling.parentNode.insertBefore(bookmarkEclipse, menuSibling);
    },
    
    /*
     * nsIDynamicContainer methods
     */
    
    onContainerMoved: function(itemId, newParent, newIndex) {
        dump("ModBookmarks.onContainerMoved(" + itemId + ", " + newParent + ", " + newIndex + ")\n");
    },
    
    onContainerNodeClosed: function(container) {
        dump("ModBookmarks.onContainerNodeClosed(" + container + ")\n");
    },
    
    onContainerNodeOpening: function(container, options) {
        dump("ModBookmarks.onContainerNodeOpening(" + container + ", " + options + ")\n");
        container.appendURINode("http://web.mit.edu", "MIT", 0, 0, null);
    },
    
    onContainerRemoving: function(itemId) {
        dump("ModBookmarks.onContainerRemoving(" + itemId + ")\n");
    },
    
    // called when bookmark menu option is selected
    bookmark: function(window, dialogFeatures) {
        //dump("bookmark(" + window + ")\n");
        var doc = window.content.document;
        if ( ! doc.location) return;
        
        Log().ok("Show add bookmark dialog " + doc.location.href);
        if ( ! dialogFeatures) {
            dialogFeatures = "centerscreen,chrome,dialog,resizable,dependent"; // XXX modal, not dependent, on Mac OS X
        }
        dialogArgs = {
            url: doc.location.href,
            name: doc.title
        };
        window.openDialog("chrome://"+EXTENSION_NAME+"/content/mod/bookmarks-add.xul",
                          "",
                          dialogFeatures,
                          dialogArgs);
    },
    
    // called when user confirms add bookmark dialog
    bookmarkInEclipse: function(url, title, resource) {
        //dump("bookmarkInEclipse(" + url + ",\n  " + title + ",\n  " + resource + ")\n");
        var project = this.datasource.GetTarget(resource, this.rdf.GetResource("http://uid.csail.mit.edu/graphspace#project"), true);
        project.QueryInterface(Ci.nsIRDFLiteral);
        var path = this.datasource.GetTarget(resource, this.rdf.GetResource("http://uid.csail.mit.edu/graphspace#path"), true);
        if (path == null) {
            path = { Value: "" }; // XXX empty string comes back as null?
        } else {
            path.QueryInterface(Ci.nsIRDFLiteral);
        }
        
        Log().ok("Add bookmark in Eclipse " + url + ", " + title + ", " + project.Value + ", " + path.Value);
        this.activator.add(this.tuples(url, title, project.Value, path.Value));
    },
    
    tuples: function(url, title, projectName, uriInProject) {
        var c = this.activator;
        var event = c.Unique("#event");
        return [
            event,
            c.Tuple(event.subject, c.URI("#action"), c.URI("#bookmark-request")),
            c.Timestamp(event.subject),
            c.Tuple(event.subject, c.URI("#url"), url),
            c.Tuple(event.subject, c.URI("#title"), title),
            c.Tuple(event.subject, c.URI("#project"), projectName),
            c.Tuple(event.subject, c.URI("#path"), uriInProject)
        ];
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports) &&
             ! iid.equals(Ci.nsIDynamicContainer)) // XXX
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component.
 */
function SingletonFactory() {
    this.instance = new ModBookmarks();
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}