/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "browse-to-docs";
const THIS_CID = Components.ID("{@MOD_BROWSE_TO_DOCS_CID@}");
const THIS_CONTRACTID = "@MOD_BROWSE_TO_DOCS_CONTRACTID@";

Data = function() { return Cc["@csail.mit.edu/codetrail/study/Data;1"].getService().wrappedJSObject; }

const StringBundleId        = "@mozilla.org/intl/stringbundle;1";
const BrowserStatusFilterId = "@mozilla.org/appshell/component/browser-status-filter;1";
const WindowMediatorId      = "@mozilla.org/appshell/window-mediator;1";

const MARKER = "mod-browse-to-docs-tab";

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

function ModBrowseToDocs() {
    this.wrappedJSObject = this;
}

/*
 * Browses to documentation.
 * see Java: codetrail.firefox.mod.BrowseToDocs
 */

ModBrowseToDocs.prototype = {
    init: function() {
        this.strings = Cc[StringBundleId]
                           .getService(Ci.nsIStringBundleService)
                               .createBundle("chrome://"+EXTENSION_NAME+"/content/mod/browse-to-docs.properties");
    },
    
    connect: function(c) {
        this.activator = c;
        c.register(this,
                   c.Tuple(null, c.URI("#action"), c.URI("#select")),
                   this.strings.GetStringFromName("graphspace-query"));
    },
    
    disconnect: function() {
        delete this.activator;
    },
    
    newWindow: function(window) {
    },
    
    notify: function(params) {
        var type = params["type"];
        if (type == "javadoc")
            this.browseFramed(params, "classFrame");
        else if (type == "eclipse")
            this.browseFramed(params, "ContentViewFrame");
        else
            this.load(params["url"]);
    },
    
    browseFramed: function(params, frame) {
        var browser = this.load(params["index"]);
        if (browser == null) return;
        var url = params["url"];
        
        var listener = {
            onStateChange: function(webProgress, request, stateFlags, status) {
                browser.webProgress.removeProgressListener(filter);
                this.browse(browser.contentWindow);
            },
            
            browse: function(window) {
                if (window.name == frame) {
                    if (window.location.href != url) {
                        window.location.href = url;
                        Data().browserEvent("browsedocs", "frame", url)
                    }
                    return true;
                }
                for (var ii = 0; ii < window.frames.length; ii++) {
                    if (this.browse(window.frames[ii]))
                        return true;
                }
                return false;
            }
        };
        var filter = Cc[BrowserStatusFilterId]
                         .createInstance(Ci.nsIWebProgress);
        filter.addProgressListener(listener, null);
        browser.webProgress.addProgressListener(filter, Ci.nsIWebProgress.NOTIFY_STATE_WINDOW);
        if ( ! browser.webProgress.isLoadingDocument) {
            listener.onStateChange(null, null, null, null);
        }
    },
    
    load: function(url) {
        //dump("load(" + url + ")\n");
        var wm = Cc[WindowMediatorId]
                     .getService(Ci.nsIWindowMediator);
        
        var tabBrowsers = wm.getEnumerator("navigator:browser");
        while (tabBrowsers.hasMoreElements()) {
            var tabBrowser = tabBrowsers.getNext().getBrowser();
            for (var index = 0; index < tabBrowser.tabContainer.childNodes.length; index++) {
                var tab = tabBrowser.tabContainer.childNodes[index];
                if (tab.hasAttribute(MARKER)) {
                    var browser = tabBrowser.getBrowserForTab(tab);
                    if (browser.contentDocument.location.href != url) {
                        browser.contentDocument.location.href = url;
                        Data().browserEvent("browsedocs", null, url);
                    }
                    return browser;
                }
            }
        }
        
        var window = wm.getMostRecentWindow("navigator:browser");
        if (window == null) return null;
        
        var tab = window.getBrowser().addTab(url);
        tab.setAttribute(MARKER, "true");
        tab.addEventListener("DOMAttrModified", this.tabIconHandler, false);
        tab.setAttribute("image", this.tabIconHandler.icon);
        var browser = window.getBrowser().getBrowserForTab(tab);
                
        var bar = window.document.createElementNS(XUL_NS, "hbox");
        bar.className = "chromeclass-status";
                
        var panel = window.document.createElementNS(XUL_NS, "statusbarpanel");
        panel.flex = 1;
        
        var label = window.document.createElementNS(XUL_NS, "label");
        label.className = "statusbarpanel-text header";
        label.setAttribute("style", "margin-top:2px; margin-bottom:3px;");
        label.flex = 1;
        label.textContent = "Eclipse selected element documentation";
        label.setAttribute("tooltiptext", "Available documentation will automatically be loaded in this browser tab");
        
        panel.appendChild(label);
        bar.appendChild(panel);
        browser.parentNode.insertBefore(bar, browser);
        
        Data().browserEvent("browsedocs", "new", url);
        
        return browser;
    },
    
    tabIconHandler: {
        icon: "chrome://"+EXTENSION_NAME+"/content/icons/eclipse.gif",
        handleEvent: function(event) {
            if (event.attrName == "src" && event.newValue != this.icon) {
                event.target.setAttribute("image", this.icon);
            }
        }
    },
    
    marshal: function() {
        return "remote:interface:codetrail.GraphSpaceListener&id:" + THIS_CONTRACTID + "\n";
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new ModBrowseToDocs();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}