/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "Eclipse resource tree RDF datasource";
const THIS_CID = Components.ID("{@MOD_RESOURCES_CID@}");
const THIS_CONTRACTID = "@MOD_RESOURCES_CONTRACTID@";

Log = function() { return Cc["@csail.mit.edu/jsjr/log;1"].getService().wrappedJSObject; }

const StringBundleId       = "@mozilla.org/intl/stringbundle;1";
const RdfServiceId         = "@mozilla.org/rdf/rdf-service;1";
const InMemoryDatasourceId = "@mozilla.org/rdf/datasource;1?name=in-memory-datasource";

/*
 * RDF datasource.
 */

function ModEclipseResources() {
    this.wrappedJSObject = this;
    this.source = Cc[InMemoryDatasourceId].createInstance(Ci.nsIRDFDataSource);
    this.source.QueryInterface(Ci.nsIRDFPurgeableDataSource);
    this.rdf = Cc[RdfServiceId].getService(Ci.nsIRDFService);

}

ModEclipseResources.prototype = {
    init: function() {
        this.strings = Cc[StringBundleId]
                           .getService(Ci.nsIStringBundleService)
                               .createBundle("chrome://"+EXTENSION_NAME+"/content/mod/eclipse-resources.properties");
    },
    
    connect: function(c) {
        //dump("Resources.connect(" + c + ")\n");
        this.activator = c;
        c.register(this,
                   c.Tuple(null, c.URI("#r6e-add"), null),
                   this.strings.GetStringFromName("graphspace-query"));
        c.register(this,
                   c.Tuple(null, c.URI("#r6e-del"), null),
                   this.strings.GetStringFromName("graphspace-query"));
        var event = c.Unique("#event");
        c.add([
            c.Tuple(event.subject, c.URI("#action"), c.URI("#resources-list-request")),
            c.Timestamp(event.subject)
        ]);
    },
    
    disconnect: function() {
        delete this.activator;
        this.source.Sweep();
    },
    
    newWindow: function(window) { },
    
    notify: function(solution) {
        //dump("Resources.notify(" + solution + ")\n");
        var parent = solution["parent"];
        var action = solution["action"].split("#r6e-")[1];
        var children = solution["children"].split("\n");
        
        var source = this.source;
        if (action == "add") {
            var fn = function(s, p, o) { source.Assert(s, p, o, true); }
        } else { // action == "del"
            var fn = source.Unassert;
        }
        
        var rdf = this.rdf;
        children.forEach(function (child) {
          uri = parent + (parent.substr(-3) == "%2F" ? "" : "%2F") + child;        
          fullpath = decodeURIComponent(uri.split("#r6e-")[1])
          project = fullpath.split("/")[1];
          relpath = fullpath.split("/").slice(2).join("/");
          fn(rdf.GetResource(uri),
             rdf.GetResource("http://uid.csail.mit.edu/graphspace#name"),
             rdf.GetLiteral(child));
          fn(rdf.GetResource(uri),
             rdf.GetResource("http://uid.csail.mit.edu/graphspace#project"),
             rdf.GetLiteral(project));
          fn(rdf.GetResource(uri),
             rdf.GetResource("http://uid.csail.mit.edu/graphspace#path"),
             rdf.GetLiteral(relpath));
          fn(rdf.GetResource(parent),
             rdf.GetResource("http://uid.csail.mit.edu/graphspace#child"),
             rdf.GetResource(uri));
        });
    },
    
    /*
     * nsIRDFDataSource methods
     */
    /*void*/ AddObserver: function(/*nsIRDFObserver*/ observer) {
        //dump("Resources.AddObserver(" + observer + ")\n");
        this.source.AddObserver(observer);
    },
    /*void*/ RemoveObserver: function(/*nsIRDFObserver*/ observer) {
        //dump("Resources.RemoveObserver(" + observer + ")\n");
        this.source.RemoveObserver(observer);
    },
    /*nsISimpleEnumerator*/ ArcLabelsIn: function(/*nsIRDFNode*/ node) {
        //dump("Resources.ArcLabelsIn(" + node + ")\n");
        return this.source.ArcLabelsIn(node);
    },
    /*nsISimpleEnumerator*/ ArcLabelsOut: function(/*nsIRDFResource*/ source) {
        //dump("Resources.ArcLabelsOut(" + source.Value + ")\n");
        return this.source.ArcLabelsOut(source)
    },
    /*void*/ Assert: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        dump("Resources.Assert(" + source.Value + ", " + property.Value + ", " + target + ", " + truthValue + ") unsupported\n");
    },
    /*void*/ beginUpdateBatch: function() {
        dump("Resources.beginUpdateBatch() unsupported\n");
    },
    /*void*/ Change: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ oldTarget, /*nsIRDFNode*/ newTarget) {
        dump("Resources.Change(" + source.Value + ", " + property.Value + ", " + oldTarget + ", " + newTarget + ") unsupported\n");
    },
    /*void*/ DoCommand: function(/*nsISupportsArray*/ sources, /*nsIRDFResource*/ command, /*nsISupportsArray*/ arguments) {
        dump("Resources.DoCommand(" + sources + ", " + command.Value + ", " + arguments + ") unsupported\n");
    },
    /*void*/ endUpdateBatch: function() {
        dump("Resources.endUpdateBatch() unsupported\n");
    },
    /*nsISimpleEnumerator*/ GetAllCmds: function(/*nsIRDFResource*/ source) {
        //dump("Resources.GetAllCmds(" + source + ")\n");
        return this.source.GetAllCmds(source);
    },
    /*nsISimpleEnumerator*/ GetAllResources: function() {
        //dump("Resources.GetAllResources()\n");
        return this.source.GetAllResources();
    },
    /*nsIRDFResource*/ GetSource: function(/*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("Resources.GetSource(" + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.GetSource(property, target, truthValue);
    },
    /*nsISimpleEnumerator*/ GetSources: function(/*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("Resources.GetSources(" + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.GetSources(property, target, truthValue);
    },
    /*nsIRDFNode*/ GetTarget: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*PRBool*/ truthValue) {
        //dump("Resources.GetTarget(" + source.Value + ", " + property.Value + ", " + truthValue + ")\n");
        return this.source.GetTarget(source, property, truthValue);
    },
    /*nsISimpleEnumerator*/ GetTargets: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*PRBool*/ truthValue) {
        //dump("Resources.GetTargets(" + source.Value + ", " + property.Value + ", " + truthValue + ")\n");
        return this.source.GetTargets(source, property, truthValue);
    },
    /*PRBool*/ hasArcIn: function(/*nsIRDFNode*/ node, /*nsIRDFResource*/ arc) {
        //dump("Resources.hasArcIn(" + node + ", " + arc.Value + ")\n");
        return this.source.hasArcIn(node, arc);
    },
    /*PRBool*/ hasArcOut: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ arc) {
        //dump("Resources.hasArcOut(" + source.Value + ", " + arc.Value + ")\n");
        return this.source.hasArcOut(source, arc);
    },
    /*PRBool*/ HasAssertion: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target, /*PRBool*/ truthValue) {
        //dump("Resources.HasAssertion(" + source.Value + ", " + property.Value + ", " + target + ", " + truthValue + ")\n");
        return this.source.HasAssertion(source, property, target, truthValue);
    },
    /*PRBool*/ IsCommandEnabled: function(/*nsISupportsArray*/ sources, /*nsIRDFResource*/ command, /*nsISupportsArray*/ arguments) {
        //dump("Resources.IsCommandEnabled(" + sources + ", " + command.Value + ", " + arguments + ")\n");
        return this.source.IsCommandEnabled(sources, command, arguments);
    },
    /*void*/ Move: function(/*nsIRDFResource*/ oldSource, /*nsIRDFResource*/ newSource, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target) {
        dump("Resources.Move(" + oldSource.Value + ", " + newSource.Value + ", " + property.Value + ", " + target + ") unsupported\n");
    },
    /*void*/ Unassert: function(/*nsIRDFResource*/ source, /*nsIRDFResource*/ property, /*nsIRDFNode*/ target) {
        dump("Resources.Unassert(" + source.Value + ", " + property.Value + ", " + target + ") unsupported\n");
    },
    
    marshal: function() {
        return "remote:interface:codetrail.GraphSpaceListener&id:" + THIS_CONTRACTID + "\n";
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports) &&
             ! iid.equals(Ci.nsIRDFDataSource))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new ModEclipseResources();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}