/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "notifier";
const THIS_CID = Components.ID("{@NOTIFIER_CID@}");
const THIS_CONTRACTID = "@NOTIFIER_CONTRACTID@";

Data = function() { return Cc["@csail.mit.edu/codetrail/study/Data;1"].getService().wrappedJSObject; }

const WindowMediatorId = "@mozilla.org/appshell/window-mediator;1"

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

/*
 * Firefox notifier.
 */

function Notifier() {
    this.wrappedJSObject = this;
}

Notifier.prototype = {
    connect: function(activator) {
        //dump("Notifier.connect(" + activator + ")\n");
        this.activator = activator;
        activator.registerAnswer(this, activator.FIREFOX);
    },
    
    disconnect: function() {
        //dump("Notifier.disconnect()\n");
        delete this.activator;
    },
    
    newWindow: function(window) { },
    
    notify: function(question) {
        var message = question["message"];
        
        var window = Cc[WindowMediatorId]
                         .getService(Ci.nsIWindowMediator)
                             .getMostRecentWindow("navigator:browser");
        var document = window.document;
        var tabBrowser = document.getElementById("content");
        
        var notebox = tabBrowser.getNotificationBox();
        var buttons = Array.prototype.map.call(question["options"].split(","),
                                               this.askUserButtonizer(question["question"]));
        var note = notebox.appendNotification(message, // removed ...
                                              "CtNotification",
                                              null,
                                              notebox.PRIORITY_INFO_MEDIUM,
                                              buttons);
        note.label = ""; // ... so we can format our message by hand
        
        // parent is an inaccessible centered hbox, so we add a spacer that jams
        // our text to the left margin where it belongs
        var spacer = document.createElementNS(XUL_NS, "spacer");
        spacer.flex = 10000;
        
        var vbox = document.createElementNS(XUL_NS, "vbox");
        vbox.flex = 1;
        
        // bump the text down by a couple of pixels to keep it off the edge
        var bump = document.createElementNS(XUL_NS, "spacer");
        bump.height = 2;
        vbox.appendChild(bump);
        
        var first = true;
        message.split("\n").forEach(function(line) {
            var desc = document.createElementNS(XUL_NS, "description");
            desc.flex = 1;
            desc.textContent = line;
            if (first) {
                desc.className = "header";
                first = false;
            }
            vbox.appendChild(desc);
        });
        
        note.insertBefore(spacer, note.firstChild);
        note.insertBefore(vbox, note.firstChild);
        
        Data().browserEvent("notify");
    },
    
    /*
     * Returns a function that can be used to map string options into buttons
     * for answering the given question id.
     */
    askUserButtonizer: function(id) {
        var activator = this.activator;
        return function(answer, index) {
            return {
                accessKey: null,
                callback: function(notification, description) {
                    activator.answer(activator.URI(id), answer);
                },
                label: answer,
                popup: null
            }
        }
    },
    
    marshal: function() {
        return "remote:interface:codetrail.GraphSpaceListener&id:" + THIS_CONTRACTID + "\n";
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    },
}

/*
 * Singleton factory for QIable component.
 */
function SingletonFactory() {
    this.instance = new Notifier();
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}