/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "field study annoyer";
const THIS_CID = Components.ID("{a4bdce70-8275-11dd-ad8b-0800200c9a66}");
const THIS_CONTRACTID = "@csail.mit.edu/codetrail/study/Annoyer;1";

const StudyDataId = "@csail.mit.edu/codetrail/study/Data;1";

const StringBundleId = "@mozilla.org/intl/stringbundle;1";
const WindowMediatorId = "@mozilla.org/appshell/window-mediator;1"
const TimerId = "@mozilla.org/timer;1";

const NOTIFICATION = "CodetrailNotification";

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

const BASELINE = 1; //0.09

function nowMin() {
    return Date.now() / 1000 / 60;
}

function Annoyer(uri, id, notification, taggers, tagdesc, sharers, sharedesc, finishers, finishdesc, closetext) {
    this.uri = uri;
    this.id = id;
    this.notification = notification;
    this.taggers = taggers;
    this.tagdesc = tagdesc;
    this.sharers = sharers;
    this.sharedesc = sharedesc;
    this.finishers = finishers;
    this.finishdesc = finishdesc;
    this.closetext = closetext;
    this.timer = Cc[TimerId].createInstance(Ci.nsITimer);
    
    this.taggers.forEach(function(b) { b.setAttribute("disabled", false); });
    this.sharers.forEach(function(b) { b.setAttribute("disabled", true); });
    this.finishers.forEach(function(b) { b.setAttribute("disabled", true); });
    
}

Annoyer.prototype = {
    tag: function(button, tag) {
        this.selectedTag = tag;
        this.tagdesc.textContent = button.getAttribute("label") + ".";
        this.taggers.forEach(function(b) { b.setAttribute("disabled", true); });
        this.sharers.forEach(function(b) { b.setAttribute("disabled", false); });
    },
    
    share: function(button, share) {
        this.selectedShare = share;
        this.sharedesc.textContent = button.getAttribute("label") + ".";
        this.sharers.forEach(function(b) { b.setAttribute("disabled", true); });
        this.finishers.forEach(function(b) { b.setAttribute("disabled", false); });
        this.countDown();
    },
    
    done: function() {
        this.timer.cancel();
        this.finishers.forEach(function(b) { b.setAttribute("disabled", true); });
        this.finishdesc.textContent = "";
        try {
            this.notification.close();
        } catch (e) { }
        this.record();
    },
    
    undo: function() {
        this.timer.cancel();
        this.tagdesc.textContent = "";
        this.sharedesc.textContent = "";
        this.finishdesc.textContent = "";
        this.finishers.forEach(function(b) { b.setAttribute("disabled", true); });
        this.taggers.forEach(function(b) { b.setAttribute("disabled", false); });
    },
    
    countDown: function() {
        var annoyer = this;
        var ii = 5;
        this.timer.init({
            observe: function(s,t,d) {
                if (--ii < 1) {
                    annoyer.done();
                } else {
                    annoyer.finishdesc.textContent = annoyer.closetext.replace("%s", ii);
                }
            }
        }, 900, Ci.nsITimer.TYPE_REPEATING_SLACK);
        this.finishdesc.textContent = this.closetext.replace("%s", ii);
    },
    
    record: function() {
        //dump("Annoyer.record() tag=" + this.selectedTag + ", share=" + this.selectedShare + "\n");
        var data = Cc[StudyDataId].getService().wrappedJSObject;
        switch (this.selectedShare) {
        case "domain":
            data.shareDomain(data.domain(this.uri));
            break;
        case "url":
            data.shareUrl(data.url(this.uri));
            break;
        }
        data.tag(this.id, this.selectedTag);
    }
}

/*
 * Field study: URL prompter.
 */

function StudyAnnoyer() {
    this.wrappedJSObject = this;
}

StudyAnnoyer.prototype = {
    init: function() {
        this.strings = Cc[StringBundleId]
                           .getService(Ci.nsIStringBundleService)
                               .createBundle("chrome://"+EXTENSION_NAME+"/locale/study/annoyer.properties");
        
        this.data = Cc[StudyDataId].getService().wrappedJSObject;
        
        this.pendingAnnoy = null;
        this.lastAnnoyMin = 0;
    },
    
    annoy: function(uri, id, tabBrowser, browser) {
        //dump("StudyWatcher.annoy(" + uri.spec + ")\n");
        this.lastAnnoyMin = nowMin();
        var document = tabBrowser.ownerDocument;
        var notebox = tabBrowser.getNotificationBox(browser);
        var note = notebox.appendNotification(uri.spec, // removed ...
                                              NOTIFICATION,
                                              null,
                                              notebox.PRIORITY_INFO_MEDIUM,
                                              [ ]);
        note.label = ""; // ... so we can format our message by hand
        
        // parent is an inaccessible centered hbox, so we add a spacer that jams
        // our text to the left margin where it belongs
        var spacer = document.createElementNS(XUL_NS, "spacer");
        spacer.flex = 10000;
        
        var vbox = document.createElementNS(XUL_NS, "vbox");
        vbox.flex = 1;
        
        // bump the text down by a couple of pixels to keep it off the edge
        var bump = document.createElementNS(XUL_NS, "spacer");
        bump.height = 2;
        vbox.appendChild(bump);
        
        var title = document.createElementNS(XUL_NS, "description");
        title.flex = 1;
        title.textContent = this.strings.GetStringFromName("browsing") + " " + uri.spec;
        title.className = "header";
        vbox.appendChild(title);
        
        var desc = document.createElementNS(XUL_NS, "description");
        desc.flex = 1;
        desc.textContent = this.strings.GetStringFromName("request");
        vbox.appendChild(desc);
        
        var space = document.createElementNS(XUL_NS, "separator");
        space.className = "thin";
        vbox.appendChild(space);
        
        var grid = document.createElementNS(XUL_NS, "grid");
        vbox.appendChild(grid);
        
        var columns = document.createElementNS(XUL_NS, "columns");
        grid.appendChild(columns);
        
        var col1 = document.createElementNS(XUL_NS, "column");
        col1.flex = 2;
        columns.appendChild(col1);
        
        var col2 = document.createElementNS(XUL_NS, "column");
        col2.flex = 3;
        columns.appendChild(col2);
        
        var col3 = document.createElementNS(XUL_NS, "column");
        col3.flex = 1;
        columns.appendChild(col3);
        
        var rows = document.createElementNS(XUL_NS, "rows");
        grid.appendChild(rows);
        
        var row1 = document.createElementNS(XUL_NS, "row");
        rows.appendChild(row1);
        
        var desc1 = document.createElementNS(XUL_NS, "hbox");
        row1.appendChild(desc1);
        
        var desc2 = document.createElementNS(XUL_NS, "hbox");
        row1.appendChild(desc2);
        
        var desc3 = document.createElementNS(XUL_NS, "hbox");
        row1.appendChild(desc3);
        
        var row2 = document.createElementNS(XUL_NS, "row");
        rows.appendChild(row2);
        
        var group1 = document.createElementNS(XUL_NS, "hbox");
        row2.appendChild(group1);
        
        var group2 = document.createElementNS(XUL_NS, "hbox");
        row2.appendChild(group2);
        
        var group3 = document.createElementNS(XUL_NS, "hbox");
        row2.appendChild(group3);
        
        var annoyer;
        
        var group1desc = document.createElementNS(XUL_NS, "description");
        group1desc.textContent = this.strings.GetStringFromName("desc-tag");
        desc1.appendChild(group1desc);
        
        var group1choice = document.createElementNS(XUL_NS, "description");
        group1choice.setAttribute("style", "margin-left:0.5em");
        desc1.appendChild(group1choice);
        
        var group2desc = document.createElementNS(XUL_NS, "description");
        group2desc.textContent = this.strings.GetStringFromName("desc-share");
        desc2.appendChild(group2desc);
        
        var group2choice = document.createElementNS(XUL_NS, "description");
        group2choice.setAttribute("style", "margin-left:0.5em");
        desc2.appendChild(group2choice);
        
        var group3desc = document.createElementNS(XUL_NS, "description");
        group3desc.textContent = this.strings.GetStringFromName("desc-done");
        desc3.appendChild(group3desc);
        
        var group3choice = document.createElementNS(XUL_NS, "description");
        group3choice.setAttribute("style", "margin-left:0.5em");
        desc3.appendChild(group3choice);
        
        var buttonDevRel = document.createElementNS(XUL_NS, "button");
        buttonDevRel.setAttribute("label", this.strings.GetStringFromName("tag-devel"));
        buttonDevRel.setAttribute("tooltiptext", this.strings.GetStringFromName("tag-devel-tip"));
        buttonDevRel.addEventListener("click", function() { annoyer.tag(buttonDevRel, "development"); }, false);
        group1.appendChild(buttonDevRel);
        
        var buttonOther = document.createElementNS(XUL_NS, "button");
        buttonOther.setAttribute("label", this.strings.GetStringFromName("tag-other"));
        buttonOther.setAttribute("tooltiptext", this.strings.GetStringFromName("tag-other-tip"));
        buttonOther.addEventListener("click", function() { annoyer.tag(buttonOther, "other"); }, false);
        group1.appendChild(buttonOther);
        
        var buttonShareAll = document.createElementNS(XUL_NS, "button");
        buttonShareAll.setAttribute("label", this.strings.GetStringFromName("share-domain").replace("%s", uri.host));
        buttonShareAll.setAttribute("tooltiptext", this.strings.GetStringFromName("share-domain-tip"));
        buttonShareAll.addEventListener("click", function() { annoyer.share(buttonShareAll, "domain"); }, false);
        group2.appendChild(buttonShareAll);
        
        var buttonShare = document.createElementNS(XUL_NS, "button");
        buttonShare.setAttribute("label", this.strings.GetStringFromName("share-url"));
        buttonShare.setAttribute("tooltiptext", this.strings.GetStringFromName("share-url-tip"));
        buttonShare.addEventListener("click", function() { annoyer.share(buttonShare, "url"); }, false);
        group2.appendChild(buttonShare);
        
        var buttonHide = document.createElementNS(XUL_NS, "button");
        buttonHide.setAttribute("label", this.strings.GetStringFromName("share-none"));
        buttonHide.setAttribute("tooltiptext", this.strings.GetStringFromName("share-none-tip"));
        buttonHide.addEventListener("click", function() { annoyer.share(buttonHide, ""); }, false);
        group2.appendChild(buttonHide);
        
        var buttonDone = document.createElementNS(XUL_NS, "button");
        buttonDone.setAttribute("label", this.strings.GetStringFromName("done"));
        buttonDone.addEventListener("click", function() { annoyer.done(); }, false);
        group3.appendChild(buttonDone);
        
        var buttonUndo = document.createElementNS(XUL_NS, "button");
        buttonUndo.setAttribute("label", this.strings.GetStringFromName("undo"));
        buttonUndo.addEventListener("click", function() { annoyer.undo(); }, false);
        group3.appendChild(buttonUndo);
        
        annoyer = new Annoyer(uri,
                              id,
                              note,
                              [ buttonDevRel, buttonOther ],
                              group1choice,
                              [ buttonShareAll, buttonShare, buttonHide ],
                              group2choice,
                              [ buttonDone, buttonUndo ],
                              group3choice,
                              this.strings.GetStringFromName("desc-close"));
        
        note.insertBefore(spacer, note.firstChild);
        note.insertBefore(vbox, note.firstChild);
    },
    
    tabLoaded: function(event) {
        if (this.pendingAnnoy && event.originalTarget.location && (event.originalTarget.location.href == this.pendingAnnoy.uri.spec)) {
            var tabBrowser = event.target;
            this.annoy(this.pendingAnnoy.uri, this.pendingAnnoy.id, tabBrowser, tabBrowser.getBrowserForDocument(event.originalTarget));
            this.pendingAnnoy = null;
        }
    },
    
    browse: function(event, uri, tooSoon, id) {
        var cutoff = BASELINE - BASELINE / (1 + nowMin() - this.lastAnnoyMin);
        if (Math.random() < cutoff) {
            if (tooSoon) {
                // MOZ e.g. Firefox 2, entries hit datasource early
                this.pendingAnnoy = { uri: uri, id: id };
            } else {
                // MOZ e.g. Firefox 3, entries hit nav history late
                var tabBrowsers = Cc[WindowMediatorId]
                                      .getService(Ci.nsIWindowMediator)
                                          .getEnumerator("navigator:browser");
                while (tabBrowsers.hasMoreElements()) {
                    var tabBrowser = tabBrowsers.getNext().getBrowser();
                    for (var index = 0; index < tabBrowser.tabContainer.childNodes.length; index++) {
                        var browser = tabBrowser.tabContainer.childNodes[index].linkedBrowser;
                        if (browser.contentDocument.location && browser.contentDocument.location.href == uri.spec) {
                            this.annoy(uri, id, tabBrowser, browser);
                        }
                    }
                }
            }
        }
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new StudyAnnoyer();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}