/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "field study first-run";
const THIS_CID = Components.ID("{6a855d8f-64dd-4af5-b391-ddb8aca43922}");
const THIS_CONTRACTID = "@csail.mit.edu/codetrail/study/FirstRun;1";

const StudyDataId = "@csail.mit.edu/codetrail/study/Data;1";

const PreferencesServicesId = "@mozilla.org/preferences-service;1";
const WindowMediatorId      = "@mozilla.org/appshell/window-mediator;1";
const UuidGenId = "@mozilla.org/uuid-generator;1";

const MARKER = "study-first-run-tab";

const START_URL = "chrome://" + EXTENSION_NAME + "/content/study/first-run.html";
const SETUP_URL = "chrome://" + EXTENSION_NAME + "/content/study/first-run-setup.html";

/*
 * Field study: first-run handler.
 */

function StudyFirstRun() {
    this.wrappedJSObject = this;
    this.connected = false;
    this.firstRunWindow = null;
}

StudyFirstRun.prototype = {
    connect: function(activator) {
        //dump("FirstRun.connect(" + activator + ")\n");
        this.connected = true;
        
        if (this.firstRunWindow != null)
            this.updateTab();
    },
    
    disconnect: function() {
        this.connected = false;
    },
    
    newWindow: function(window) {
        //dump("FirstRun.newWindow(" + window + ")\n");
        var firstrun = this;
        window.addEventListener("load", function (e) { firstrun.windowLoaded(window) }, false);
    },
    
    windowLoaded: function(window) {
        var installed = false;
        try {
            installed = this.prefBranch().getBoolPref("installed");
        } catch (e) { }
        if (installed) return;
        if (this.firstRunWindow != null) return;
        this.firstRunWindow = window;
        
        var firstRun = this;
        window.setTimeout(function() {
            browser = window.getBrowser();
            var tab = browser.addTab(firstRun.connected ? SETUP_URL : START_URL);
            tab.setAttribute(MARKER, "true");
            browser.selectedTab = tab;
        }, 100);
    },
    
    updateTab: function() {
        var installed = false;
        try {
            installed = this.prefBranch().getBoolPref("installed");
        } catch (e) { }
        if (installed) return;
                
        var tabBrowser = this.firstRunWindow.getBrowser();
        this.firstRunWindow.setTimeout(function() {
            for (var index = 0; index < tabBrowser.tabContainer.childNodes.length; index++) {
                var tab = tabBrowser.tabContainer.childNodes[index];
                if (tab.hasAttribute(MARKER)) {
                    var browser = tabBrowser.getBrowserForTab(tab);
                    browser.contentDocument.location.href = SETUP_URL;
                    return;
                }
            }
            var wm = Cc[WindowMediatorId]
                         .getService(Ci.nsIWindowMediator);
            var window = wm.getMostRecentWindow("navigator:browser");
            if (window == null) return;
            
            window.getBrowser().addTab(SETUP_URL);
        }, 100);
    },
    
    enable: function(source) {
        //dump("FirstRun.enable(" + email + ")\n");
        this.prefBranch().setBoolPref("installed", true);
        var data = Cc[StudyDataId].getService().wrappedJSObject;
        data.initEnabled();
        
        data.source("random", Cc[UuidGenId].getService(Ci.nsIUUIDGenerator).generateUUID().toString());
        for (field in source) {
            data.source(field, source[field]);
        }
        
        data.submit({
            ok: "Codetrail data submission test succeeded.",
            fail: "Codetrail data submission test failed. Your data might not be submitted."
        });
        
        return "chrome://" + EXTENSION_NAME + "/content/study/first-run-done.html"
    },
    
    prefBranch: function() {
        return Cc[PreferencesServicesId]
                   .getService(Ci.nsIPrefService)
                       .getBranch(EXTENSION_NAME + ".");
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component.
 */
function SingletonFactory() {
    this.instance = new StudyFirstRun();
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}