/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "field study watcher";
const THIS_CID = Components.ID("{8fee74eb-32ea-45e0-abd0-65d561bfbc96}");
const THIS_CONTRACTID = "@csail.mit.edu/codetrail/study/Watcher;1";

const StudyDataId = "@csail.mit.edu/codetrail/study/Data;1";
// DISABLED const StudyAnnoyerId = "@csail.mit.edu/codetrail/study/Annoyer;1";

const IOServiceId = "@mozilla.org/network/io-service;1";
const RdfServiceId = "@mozilla.org/rdf/rdf-service;1";
const HistoryId = "@mozilla.org/rdf/datasource;1?name=history";
const NavHistoryServiceId = "@mozilla.org/browser/nav-history-service;1";
const WindowMediatorId = "@mozilla.org/appshell/window-mediator;1"
    
const TAB_MARKER = "mod-browse-to-docs-tab"; // XXX must match def'n in ModBrowseToDocs

/*
 * Field study: browsing watcher.
 */

function StudyWatcher() {
    this.wrappedJSObject = this;
}

StudyWatcher.prototype = {
    init: function() {
        this.io = Cc[IOServiceId].getService(Ci.nsIIOService);
        
        this.rdf = Cc[RdfServiceId].getService(Ci.nsIRDFService);
        this.rdfHistory = this.rdf.GetResource("NC:HistoryRoot");
        this.rdfChild = this.rdf.GetResource("http://home.netscape.com/NC-rdf#child");
        this.rdfName = this.rdf.GetResource("http://home.netscape.com/NC-rdf#Name");
        
        this.data = Cc[StudyDataId].getService().wrappedJSObject;
        this.annoyer = null; // DISABLED Cc[StudyAnnoyerId].getService().wrappedJSObject;
        
        if (Cc[HistoryId]) {
            // MOZ Firefox 2
            Cc[HistoryId].getService(Ci.nsIRDFDataSource).AddObserver(this);
        } else {
            // MOZ Firefox 3
            Cc[NavHistoryServiceId].getService(Ci.nsINavHistoryService).addObserver(this, false);
        }
    },
    
    connect: function(activator) { },
    
    disconnect: function() { },
    
    newWindow: function(window) {
        //dump("StudyWatcher.newWindow(" + window + ")\n");
        var watcher = this;
        window.addEventListener("load", function (e) { watcher.windowLoaded(window) }, false);
        
        var count = function() {
            var count = 0;
            var tabBrowsers = Cc[WindowMediatorId]
                                  .getService(Ci.nsIWindowMediator)
                                      .getEnumerator("navigator:browser");
            while (tabBrowsers.hasMoreElements()) {
                tabBrowsers.getNext();
                count++;
            }
            return count;
        }();
        
        this.data.browserEvent("window open", count);
        window.addEventListener("focus", function(event) {
            if (event.originalTarget.nodeName && event.originalTarget.nodeName != "#document") return;
            watcher.data.browserFocus();
        }, false);
        window.addEventListener("close", function(e) { watcher.data.browserEvent("window close"); }, false);
    },
    
    windowLoaded: function(window) {
        //dump("StudyWatcher.windowLoaded(" + window + ")\n");
        var watcher = this;
        window.getBrowser().addEventListener("load", function (e) { watcher.tabLoaded(e) }, true);
        
        var tabs = window.getBrowser().tabContainer;
        tabs.addEventListener("TabOpen", function (e) { watcher.data.browserEvent("tab open", tabs.childNodes.length); }, false);
        tabs.addEventListener("TabMove", function (e) { watcher.data.browserEvent("tab move"); }, false);
        tabs.addEventListener("TabSelect", function (e) { watcher.tabSelected(e); }, false);
        tabs.addEventListener("TabClose", function (e) { watcher.data.browserEvent("tab close", tabs.childNodes.length); }, false);
    },
    
    tabSelected: function (event) {
        if (event.target.hasAttribute(TAB_MARKER))
            this.data.browserEvent("tab select", "docs");
        else
            this.data.browserEvent("tab select");
    },
    
    tabLoaded: function(event) {
        //dump("StudyWatcher.tabLoaded(...)\n");
        if (event.originalTarget.nodeName != "#document") return;
        if (this.annoyer) {
            this.annoyer.tabLoaded(event);
        }
    },
    
    browse: function(event, uri, tooSoon) {
        //dump("StudyWatcher.browse(" + event + ", " + uri.spec + ", " + tooSoon + ")\n");
        var id = this.data.browse(event, uri);
        if (id <= 0) return; // already shared
        if (uri.scheme == "chrome" || uri.scheme == "file") return; // never share
        if (this.annoyer) {
            this.annoyer.browse(event, uri, tooSoon, id);
        }
    },
    
    /*
     * nsIRDFObserver methods
     * MOZ used in Firefox 2
     */
    onAssert: function(dataSource, source, property, target) {
        if (source.EqualsString("NC:HistoryRoot")) {
            this.historyNode = target
        } else if (this.historyNode
                   && source.EqualsNode(this.historyNode)
                   && property.EqualsString("http://home.netscape.com/NC-rdf#Name")
                   && (target instanceof Ci.nsIRDFLiteral)) {
            this.browse("browse", this.io.newURI(this.historyNode.Value, null, null), true);
        }
    },
    onBeginUpdateBatch: function(dataSource) { },
    onChange: function(dataSource, source, property, oldTarget, newTarget) {
        if (property.EqualsString("http://home.netscape.com/NC-rdf#Date")
                && dataSource.HasAssertion(this.rdfHistory, this.rdfChild, source, true)) {
            this.browse("browse", this.io.newURI(source.Value, null, null), true);
        }
    },
    onEndUpdateBatch: function(dataSource) { },
    onMove: function(dataSource, oldSource, newSource, property, target) { },
    onUnassert: function(dataSource, source, property, target) { },
    
    /*
     * nsINavHistoryObserver methods
     * MOZ used in Firefox 3
     */
    onBeginUpdateBatch: function() { },
    onClearHistory: function() { },
    onDeleteURI: function(/*nsIURI*/ uri) { },
    onEndUpdateBatch: function() { },
    onPageChanged: function(/*nsIURI*/ uri, /*PRUint32*/ what, /*AString*/ value) { },
    onPageExpired: function(/*nsIURI*/ uri, /*PRTime*/ visitTime, /*PRBool*/ wholeEntry) { },
    onTitleChanged: function(/*nsIURI*/ uri, /*AString*/ pageTitle) { },
    onVisit: function(/*nsIURI*/ uri, /*int*/ visitID, /*PRTime*/ time, /*int*/ sessionID, /*int*/ referringID, /*int*/ transitionType) {
        switch (transitionType) {
        case Ci.nsINavHistoryService.TRANSITION_EMBED:
            return;
        case Ci.nsINavHistoryService.TRANSITION_LINK:
            var event = "browse link";
            break;
        case Ci.nsINavHistoryService.TRANSITION_TYPED:
            var event = "browse url";
            break;
        case Ci.nsINavHistoryService.TRANSITION_BOOKMARK:
            var event = "browse bookmark";
            break;
        case Ci.nsINavHistoryService.TRANSITION_DOWNLOAD:
            var event = "browse download";
            break;
        case Ci.nsINavHistoryService.TRANSITION_REDIRECT_PERMANENT:
        case Ci.nsINavHistoryService.TRANSITION_REDIRECT_TEMPORARY:
            var event = "browse redir";
            break;
        default:
            var event = "browse";
        }
        this.browse(event, uri, false);
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports) &&
             ! iid.equals(Ci.nsIRDFObserver))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component, with initialization.
 */
function SingletonFactory() {
    this.instance = null;
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    if (this.instance == null) {
        this.instance = new StudyWatcher();
        this.instance.init();
    }
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}