/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

const EXTENSION_NAME = "@NAME@";
const EXTENSION_HUMAN_NAME = "@HUMAN_NAME@";

const THIS_DESCRIPTION = "watcher";
const THIS_CID = Components.ID("{@WATCHER_CID@}");
const THIS_CONTRACTID = "@WATCHER_CONTRACTID@";

const RdfServiceId = "@mozilla.org/rdf/rdf-service;1";
const HistoryId = "@mozilla.org/rdf/datasource;1?name=history";
const NavHistoryServiceId = "@mozilla.org/browser/nav-history-service;1";

/*
 * Firefox watcher.
 */

function Watcher() {
    this.wrappedJSObject = this;
    
    this.rdf = Cc[RdfServiceId].getService(Ci.nsIRDFService);
    this.rdfHistory = this.rdf.GetResource("NC:HistoryRoot");
    this.rdfChild = this.rdf.GetResource("http://home.netscape.com/NC-rdf#child");
    this.rdfName = this.rdf.GetResource("http://home.netscape.com/NC-rdf#Name");
}

Watcher.prototype = {
    connect: function(activator) {
        //dump("Watcher.connect(" + activator + ")\n");
        this.activator = activator;
    },
    
    disconnect: function() {
        delete this.activator;
    },
    
    newWindow: function(window) {
        //dump("Watcher.newWindow(" + window + ")\n");
        var watcher = this;
        window.addEventListener("load", function (e) { watcher.windowLoaded(window) }, false);
    },
    
    windowLoaded: function(window) {
        if (Cc[HistoryId]) {
            // MOZ Firefox 2
            Cc[HistoryId].getService(Ci.nsIRDFDataSource).AddObserver(this);
        } else {
            // MOZ Firefox 3
            this.navHistory = Cc[NavHistoryServiceId].getService(Ci.nsINavHistoryService);
            this.navHistory.addObserver(this, false);
        }
        
        var content = window.document.getElementById("appcontent");
        var watcher = this;
        content.addEventListener("load", function(e) { watcher.webPageLoaded(e) }, true);
        
        // add a progress listener that requests no notifications
        // the browser will check to see that we have an onLinkIconAvailable function
        // XXX could use the progress listener instead of load listener above
        window.getBrowser().addProgressListener(this, 0);
    },
    
    webPageLoaded: function(event) {
        if ( ! this.activator) return;
        var doc = event.originalTarget;
        if ( ! doc.location) return;
        this.activator.add(this.goTuples("load", doc.location.href, doc.title));
        
        var elts = doc.evaluate("//pre", doc, null, 0, null);
        var elt;
        while (elt = elts.iterateNext()) {
            this.activator.add(this.snippetTuples(doc.location.href, doc.title, elt.textContent));
        }
    },
    
    /*
     * nsIWebProgressListener methods
     */
    onLocationChange: function(webProgress, request, location) { },
    onProgressChange: function(webProgress, request, curSelfProgress, maxSelfProgress, curTotalProgress, maxTotalProgress) { },
    onSecurityChange: function(webProgress, request, state) { },
    onStateChange: function(webProgress, request, stateFlags, status ) { },
    onStatusChange: function(webProgress, request, status, message ) { },
    
    /*
     * not a nsIWebProgressListener method
     */
    onLinkIconAvailable: function(browser) {
        if ( ! this.activator) return;
        //dump("onLinkIconAvailable(" + browser + ")\n");
        if ( ! browser.mIconURL) return;
        if ( ! browser.contentDocument.location) return;
        this.activator.add(this.faviconTuples(browser.contentDocument.location.href, browser.mIconURL));
    },
    
    /*
     * nsIRDFObserver methods
     * MOZ used in Firefox 2
     */
    onAssert: function(dataSource, source, property, target) {
        if ( ! this.activator) return;
        if (source.EqualsString("NC:HistoryRoot")) {
            this.historyNode = target
        } else if (this.historyNode
                   && source.EqualsNode(this.historyNode)
                   && property.EqualsString("http://home.netscape.com/NC-rdf#Name")
                   && (target instanceof Ci.nsIRDFLiteral)) {
            this.activator.add(this.goTuples("browse", this.historyNode.Value, target.Value));
        }
    },
    onBeginUpdateBatch: function(dataSource) { },
    onChange: function(dataSource, source, property, oldTarget, newTarget) {
        if ( ! this.activator) return;
        if (property.EqualsString("http://home.netscape.com/NC-rdf#Date")
                && dataSource.HasAssertion(this.rdfHistory, this.rdfChild, source, true)) {
            titleNode = dataSource.GetTarget(source, this.rdfName, true);
            titleNode.QueryInterface(Ci.nsIRDFLiteral);
            this.activator.add(this.goTuples("browse", source.Value, titleNode.Value));
        }
    },
    onEndUpdateBatch: function(dataSource) { },
    onMove: function(dataSource, oldSource, newSource, property, target) { },
    onUnassert: function(dataSource, source, property, target) { },
    
    /*
     * nsINavHistoryObserver methods
     * MOZ used in Firefox 3
     */
    onBeginUpdateBatch: function() { },
    onClearHistory: function() { },
    onDeleteURI: function(/*nsIURI*/ URI) { },
    onEndUpdateBatch: function() { },
    onPageChanged: function(/*nsIURI*/ URI, /*PRUint32*/ what, /*AString*/ value) { },
    onPageExpired: function(/*nsIURI*/ URI, /*PRTime*/ visitTime, /*PRBool*/ wholeEntry) { },
    onTitleChanged: function(/*nsIURI*/ URI, /*AString*/ pageTitle) { },
    onVisit: function(/*nsIURI*/ URI, /*int*/ visitID, /*PRTime*/ time, /*int*/ sessionID, /*int*/ referringID, /*int*/ transitionType) {
        if ( ! this.activator) return;
        if (transitionType == Ci.nsINavHistoryService.TRANSITION_EMBED) return;
        this.activator.add(this.goTuples("browse", URI.spec, this.navHistory.getPageTitle(URI)));
    },
    
    /*
     * Tuples
     */
    
    goTuples: function(action, url, title) {
        var c = this.activator;
        var event = c.Unique("#event");
        var urlUri = c.URI("#" + encodeURIComponent(url));
        return [
            event,
            c.Tuple(event.subject, c.URI("#action"), c.URI("#" + action)),
            c.Tuple(event.subject, c.URI("#source"), c.FIREFOX),
            c.Timestamp(event.subject),
            c.Tuple(event.subject, c.URI("#artifact"), urlUri),
            c.Tuple(urlUri, c.URI("#url"), url),
            c.Tuple(urlUri, c.URI("#title"), title) // XXX problem if title changes
        ];
    },
    
    faviconTuples: function(url, iconUrl) {
        var c = this.activator;
        return [
            c.Tuple(c.URI("#" + encodeURIComponent(url)), c.URI("#icon"), iconUrl)
        ];
    },
    
    snippetTuples: function(url, title, text) {
        var c = this.activator;
        var urlUri = c.URI("#" + encodeURIComponent(url));
        return [
            c.Tuple(c.URI("#" + encodeURIComponent(url)), c.URI("#snippet"), text),
        ];
    },
    
    QueryInterface: function(iid) {
        if ( ! iid.equals(Ci.nsISupports) &&
             ! iid.equals(Ci.nsISupportsWeakReference) &&
             ! iid.equals(Ci.nsIWebProgressListener) &&
             ! iid.equals(Ci.nsIRDFObserver))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}

/*
 * Singleton factory for QIable component.
 */
function SingletonFactory() {
    this.instance = new Watcher();
}

SingletonFactory.prototype.createInstance = function(outer, iid) {
    if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
    
    return this.instance.QueryInterface(iid);
}

const THE_FACTORY = new SingletonFactory();

/*
 * Module automagically loaded by Firefox.
 */

var Module = new Object();

Module.registerSelf = function(compMgr, fileSpec, location, type) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(THIS_CID,
                                    EXTENSION_HUMAN_NAME + " " + THIS_DESCRIPTION,
                                    THIS_CONTRACTID,
                                    fileSpec, location, type);
}

Module.unregisterSelf = function(compMgr, fileSpec, location) {
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.unregisterFactoryLocation(THIS_CID, fileSpec);
}

Module.getClassObject = function(compMgr, cid, iid) {
    if ( ! iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (cid.equals(THIS_CID))
        return THE_FACTORY;

    throw Components.results.NS_ERROR_NO_INTERFACE;
}

Module.canUnload = function(compMgr) {
    return true;
}

function NSGetModule(compMgr, fileSpec) {
    return Module;
}