/*
 * Codetrail
 * 
 * Copyright (c) 2007-2008 Massachusetts Institute of Technology
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package jsjr;

import java.io.IOException;

/**
 * Remote method invocation connector using a simple stack-oriented plain-text protocol.
 * 
 * @author maxg
 */
public interface Connector {
    
    /**
     * Opens this <code>Connector</code> for connections.
     */
    public void accept();
    
    /**
     * Close this <code>Connector</code>.
     * 
     * @throws IOException if the socket fails to close
     */
    public void close() throws IOException;
    
    /**
     * Add a connection listener.
     * 
     * @param listener listener to add
     */
    public void addConnectionListener(ConnectionListener listener);
    
    /**
     * Export the given object so that it can receive remote method calls
     * targeted at the given ID.
     * 
     * @param id remote object ID
     * @param object object to receive remote method calls
     */
    public void export(String id, Object object);
    
}
